import gym

if __name__ == '__main__':
    env = gym.make('gym_backgammon:backgammon-v0')
    assert env.observation_space.low[0] == 0.0 and env.observation_space.high[0] == 1.0
    assert env.observation_space.low[1] == 0.0 and env.observation_space.high[1] == 1.0
    assert env.observation_space.low[2] == 0.0 and env.observation_space.high[2] == 1.0
    assert env.observation_space.low[3] == 0.0 and env.observation_space.high[3] == 6.0
    assert env.observation_space.low[4] == 0.0 and env.observation_space.high[4] == 1.0
    assert env.observation_space.low[5] == 0.0 and env.observation_space.high[5] == 1.0
    assert env.observation_space.low[6] == 0.0 and env.observation_space.high[6] == 1.0
    assert env.observation_space.low[7] == 0.0 and env.observation_space.high[7] == 6.0
    assert env.observation_space.low[8] == 0.0 and env.observation_space.high[8] == 1.0
    assert env.observation_space.low[9] == 0.0 and env.observation_space.high[9] == 1.0
    assert env.observation_space.low[10] == 0.0 and env.observation_space.high[10] == 1.0
    assert env.observation_space.low[11] == 0.0 and env.observation_space.high[11] == 6.0
    assert env.observation_space.low[12] == 0.0 and env.observation_space.high[12] == 1.0
    assert env.observation_space.low[13] == 0.0 and env.observation_space.high[13] == 1.0
    assert env.observation_space.low[14] == 0.0 and env.observation_space.high[14] == 1.0
    assert env.observation_space.low[15] == 0.0 and env.observation_space.high[15] == 6.0
    assert env.observation_space.low[16] == 0.0 and env.observation_space.high[16] == 1.0
    assert env.observation_space.low[17] == 0.0 and env.observation_space.high[17] == 1.0
    assert env.observation_space.low[18] == 0.0 and env.observation_space.high[18] == 1.0
    assert env.observation_space.low[19] == 0.0 and env.observation_space.high[19] == 6.0
    assert env.observation_space.low[20] == 0.0 and env.observation_space.high[20] == 1.0
    assert env.observation_space.low[21] == 0.0 and env.observation_space.high[21] == 1.0
    assert env.observation_space.low[22] == 0.0 and env.observation_space.high[22] == 1.0
    assert env.observation_space.low[23] == 0.0 and env.observation_space.high[23] == 6.0
    assert env.observation_space.low[24] == 0.0 and env.observation_space.high[24] == 1.0
    assert env.observation_space.low[25] == 0.0 and env.observation_space.high[25] == 1.0
    assert env.observation_space.low[26] == 0.0 and env.observation_space.high[26] == 1.0
    assert env.observation_space.low[27] == 0.0 and env.observation_space.high[27] == 6.0
    assert env.observation_space.low[28] == 0.0 and env.observation_space.high[28] == 1.0
    assert env.observation_space.low[29] == 0.0 and env.observation_space.high[29] == 1.0
    assert env.observation_space.low[30] == 0.0 and env.observation_space.high[30] == 1.0
    assert env.observation_space.low[31] == 0.0 and env.observation_space.high[31] == 6.0
    assert env.observation_space.low[32] == 0.0 and env.observation_space.high[32] == 1.0
    assert env.observation_space.low[33] == 0.0 and env.observation_space.high[33] == 1.0
    assert env.observation_space.low[34] == 0.0 and env.observation_space.high[34] == 1.0
    assert env.observation_space.low[35] == 0.0 and env.observation_space.high[35] == 6.0
    assert env.observation_space.low[36] == 0.0 and env.observation_space.high[36] == 1.0
    assert env.observation_space.low[37] == 0.0 and env.observation_space.high[37] == 1.0
    assert env.observation_space.low[38] == 0.0 and env.observation_space.high[38] == 1.0
    assert env.observation_space.low[39] == 0.0 and env.observation_space.high[39] == 6.0
    assert env.observation_space.low[40] == 0.0 and env.observation_space.high[40] == 1.0
    assert env.observation_space.low[41] == 0.0 and env.observation_space.high[41] == 1.0
    assert env.observation_space.low[42] == 0.0 and env.observation_space.high[42] == 1.0
    assert env.observation_space.low[43] == 0.0 and env.observation_space.high[43] == 6.0
    assert env.observation_space.low[44] == 0.0 and env.observation_space.high[44] == 1.0
    assert env.observation_space.low[45] == 0.0 and env.observation_space.high[45] == 1.0
    assert env.observation_space.low[46] == 0.0 and env.observation_space.high[46] == 1.0
    assert env.observation_space.low[47] == 0.0 and env.observation_space.high[47] == 6.0
    assert env.observation_space.low[48] == 0.0 and env.observation_space.high[48] == 1.0
    assert env.observation_space.low[49] == 0.0 and env.observation_space.high[49] == 1.0
    assert env.observation_space.low[50] == 0.0 and env.observation_space.high[50] == 1.0
    assert env.observation_space.low[51] == 0.0 and env.observation_space.high[51] == 6.0
    assert env.observation_space.low[52] == 0.0 and env.observation_space.high[52] == 1.0
    assert env.observation_space.low[53] == 0.0 and env.observation_space.high[53] == 1.0
    assert env.observation_space.low[54] == 0.0 and env.observation_space.high[54] == 1.0
    assert env.observation_space.low[55] == 0.0 and env.observation_space.high[55] == 6.0
    assert env.observation_space.low[56] == 0.0 and env.observation_space.high[56] == 1.0
    assert env.observation_space.low[57] == 0.0 and env.observation_space.high[57] == 1.0
    assert env.observation_space.low[58] == 0.0 and env.observation_space.high[58] == 1.0
    assert env.observation_space.low[59] == 0.0 and env.observation_space.high[59] == 6.0
    assert env.observation_space.low[60] == 0.0 and env.observation_space.high[60] == 1.0
    assert env.observation_space.low[61] == 0.0 and env.observation_space.high[61] == 1.0
    assert env.observation_space.low[62] == 0.0 and env.observation_space.high[62] == 1.0
    assert env.observation_space.low[63] == 0.0 and env.observation_space.high[63] == 6.0
    assert env.observation_space.low[64] == 0.0 and env.observation_space.high[64] == 1.0
    assert env.observation_space.low[65] == 0.0 and env.observation_space.high[65] == 1.0
    assert env.observation_space.low[66] == 0.0 and env.observation_space.high[66] == 1.0
    assert env.observation_space.low[67] == 0.0 and env.observation_space.high[67] == 6.0
    assert env.observation_space.low[68] == 0.0 and env.observation_space.high[68] == 1.0
    assert env.observation_space.low[69] == 0.0 and env.observation_space.high[69] == 1.0
    assert env.observation_space.low[70] == 0.0 and env.observation_space.high[70] == 1.0
    assert env.observation_space.low[71] == 0.0 and env.observation_space.high[71] == 6.0
    assert env.observation_space.low[72] == 0.0 and env.observation_space.high[72] == 1.0
    assert env.observation_space.low[73] == 0.0 and env.observation_space.high[73] == 1.0
    assert env.observation_space.low[74] == 0.0 and env.observation_space.high[74] == 1.0
    assert env.observation_space.low[75] == 0.0 and env.observation_space.high[75] == 6.0
    assert env.observation_space.low[76] == 0.0 and env.observation_space.high[76] == 1.0
    assert env.observation_space.low[77] == 0.0 and env.observation_space.high[77] == 1.0
    assert env.observation_space.low[78] == 0.0 and env.observation_space.high[78] == 1.0
    assert env.observation_space.low[79] == 0.0 and env.observation_space.high[79] == 6.0
    assert env.observation_space.low[80] == 0.0 and env.observation_space.high[80] == 1.0
    assert env.observation_space.low[81] == 0.0 and env.observation_space.high[81] == 1.0
    assert env.observation_space.low[82] == 0.0 and env.observation_space.high[82] == 1.0
    assert env.observation_space.low[83] == 0.0 and env.observation_space.high[83] == 6.0
    assert env.observation_space.low[84] == 0.0 and env.observation_space.high[84] == 1.0
    assert env.observation_space.low[85] == 0.0 and env.observation_space.high[85] == 1.0
    assert env.observation_space.low[86] == 0.0 and env.observation_space.high[86] == 1.0
    assert env.observation_space.low[87] == 0.0 and env.observation_space.high[87] == 6.0
    assert env.observation_space.low[88] == 0.0 and env.observation_space.high[88] == 1.0
    assert env.observation_space.low[89] == 0.0 and env.observation_space.high[89] == 1.0
    assert env.observation_space.low[90] == 0.0 and env.observation_space.high[90] == 1.0
    assert env.observation_space.low[91] == 0.0 and env.observation_space.high[91] == 6.0
    assert env.observation_space.low[92] == 0.0 and env.observation_space.high[92] == 1.0
    assert env.observation_space.low[93] == 0.0 and env.observation_space.high[93] == 1.0
    assert env.observation_space.low[94] == 0.0 and env.observation_space.high[94] == 1.0
    assert env.observation_space.low[95] == 0.0 and env.observation_space.high[95] == 6.0
    assert env.observation_space.low[96] == 0.0 and env.observation_space.high[96] == 7.5
    assert env.observation_space.low[97] == 0.0 and env.observation_space.high[97] == 1.0
    assert env.observation_space.low[98] == 0.0 and env.observation_space.high[98] == 1.0
    assert env.observation_space.low[99] == 0.0 and env.observation_space.high[99] == 1.0
    assert env.observation_space.low[100] == 0.0 and env.observation_space.high[100] == 1.0
    assert env.observation_space.low[101] == 0.0 and env.observation_space.high[101] == 6.0
    assert env.observation_space.low[102] == 0.0 and env.observation_space.high[102] == 1.0
    assert env.observation_space.low[103] == 0.0 and env.observation_space.high[103] == 1.0
    assert env.observation_space.low[104] == 0.0 and env.observation_space.high[104] == 1.0
    assert env.observation_space.low[105] == 0.0 and env.observation_space.high[105] == 6.0
    assert env.observation_space.low[106] == 0.0 and env.observation_space.high[106] == 1.0
    assert env.observation_space.low[107] == 0.0 and env.observation_space.high[107] == 1.0
    assert env.observation_space.low[108] == 0.0 and env.observation_space.high[108] == 1.0
    assert env.observation_space.low[109] == 0.0 and env.observation_space.high[109] == 6.0
    assert env.observation_space.low[110] == 0.0 and env.observation_space.high[110] == 1.0
    assert env.observation_space.low[111] == 0.0 and env.observation_space.high[111] == 1.0
    assert env.observation_space.low[112] == 0.0 and env.observation_space.high[112] == 1.0
    assert env.observation_space.low[113] == 0.0 and env.observation_space.high[113] == 6.0
    assert env.observation_space.low[114] == 0.0 and env.observation_space.high[114] == 1.0
    assert env.observation_space.low[115] == 0.0 and env.observation_space.high[115] == 1.0
    assert env.observation_space.low[116] == 0.0 and env.observation_space.high[116] == 1.0
    assert env.observation_space.low[117] == 0.0 and env.observation_space.high[117] == 6.0
    assert env.observation_space.low[118] == 0.0 and env.observation_space.high[118] == 1.0
    assert env.observation_space.low[119] == 0.0 and env.observation_space.high[119] == 1.0
    assert env.observation_space.low[120] == 0.0 and env.observation_space.high[120] == 1.0
    assert env.observation_space.low[121] == 0.0 and env.observation_space.high[121] == 6.0
    assert env.observation_space.low[122] == 0.0 and env.observation_space.high[122] == 1.0
    assert env.observation_space.low[123] == 0.0 and env.observation_space.high[123] == 1.0
    assert env.observation_space.low[124] == 0.0 and env.observation_space.high[124] == 1.0
    assert env.observation_space.low[125] == 0.0 and env.observation_space.high[125] == 6.0
    assert env.observation_space.low[126] == 0.0 and env.observation_space.high[126] == 1.0
    assert env.observation_space.low[127] == 0.0 and env.observation_space.high[127] == 1.0
    assert env.observation_space.low[128] == 0.0 and env.observation_space.high[128] == 1.0
    assert env.observation_space.low[129] == 0.0 and env.observation_space.high[129] == 6.0
    assert env.observation_space.low[130] == 0.0 and env.observation_space.high[130] == 1.0
    assert env.observation_space.low[131] == 0.0 and env.observation_space.high[131] == 1.0
    assert env.observation_space.low[132] == 0.0 and env.observation_space.high[132] == 1.0
    assert env.observation_space.low[133] == 0.0 and env.observation_space.high[133] == 6.0
    assert env.observation_space.low[134] == 0.0 and env.observation_space.high[134] == 1.0
    assert env.observation_space.low[135] == 0.0 and env.observation_space.high[135] == 1.0
    assert env.observation_space.low[136] == 0.0 and env.observation_space.high[136] == 1.0
    assert env.observation_space.low[137] == 0.0 and env.observation_space.high[137] == 6.0
    assert env.observation_space.low[138] == 0.0 and env.observation_space.high[138] == 1.0
    assert env.observation_space.low[139] == 0.0 and env.observation_space.high[139] == 1.0
    assert env.observation_space.low[140] == 0.0 and env.observation_space.high[140] == 1.0
    assert env.observation_space.low[141] == 0.0 and env.observation_space.high[141] == 6.0
    assert env.observation_space.low[142] == 0.0 and env.observation_space.high[142] == 1.0
    assert env.observation_space.low[143] == 0.0 and env.observation_space.high[143] == 1.0
    assert env.observation_space.low[144] == 0.0 and env.observation_space.high[144] == 1.0
    assert env.observation_space.low[145] == 0.0 and env.observation_space.high[145] == 6.0
    assert env.observation_space.low[146] == 0.0 and env.observation_space.high[146] == 1.0
    assert env.observation_space.low[147] == 0.0 and env.observation_space.high[147] == 1.0
    assert env.observation_space.low[148] == 0.0 and env.observation_space.high[148] == 1.0
    assert env.observation_space.low[149] == 0.0 and env.observation_space.high[149] == 6.0
    assert env.observation_space.low[150] == 0.0 and env.observation_space.high[150] == 1.0
    assert env.observation_space.low[151] == 0.0 and env.observation_space.high[151] == 1.0
    assert env.observation_space.low[152] == 0.0 and env.observation_space.high[152] == 1.0
    assert env.observation_space.low[153] == 0.0 and env.observation_space.high[153] == 6.0
    assert env.observation_space.low[154] == 0.0 and env.observation_space.high[154] == 1.0
    assert env.observation_space.low[155] == 0.0 and env.observation_space.high[155] == 1.0
    assert env.observation_space.low[156] == 0.0 and env.observation_space.high[156] == 1.0
    assert env.observation_space.low[157] == 0.0 and env.observation_space.high[157] == 6.0
    assert env.observation_space.low[158] == 0.0 and env.observation_space.high[158] == 1.0
    assert env.observation_space.low[159] == 0.0 and env.observation_space.high[159] == 1.0
    assert env.observation_space.low[160] == 0.0 and env.observation_space.high[160] == 1.0
    assert env.observation_space.low[161] == 0.0 and env.observation_space.high[161] == 6.0
    assert env.observation_space.low[162] == 0.0 and env.observation_space.high[162] == 1.0
    assert env.observation_space.low[163] == 0.0 and env.observation_space.high[163] == 1.0
    assert env.observation_space.low[164] == 0.0 and env.observation_space.high[164] == 1.0
    assert env.observation_space.low[165] == 0.0 and env.observation_space.high[165] == 6.0
    assert env.observation_space.low[166] == 0.0 and env.observation_space.high[166] == 1.0
    assert env.observation_space.low[167] == 0.0 and env.observation_space.high[167] == 1.0
    assert env.observation_space.low[168] == 0.0 and env.observation_space.high[168] == 1.0
    assert env.observation_space.low[169] == 0.0 and env.observation_space.high[169] == 6.0
    assert env.observation_space.low[170] == 0.0 and env.observation_space.high[170] == 1.0
    assert env.observation_space.low[171] == 0.0 and env.observation_space.high[171] == 1.0
    assert env.observation_space.low[172] == 0.0 and env.observation_space.high[172] == 1.0
    assert env.observation_space.low[173] == 0.0 and env.observation_space.high[173] == 6.0
    assert env.observation_space.low[174] == 0.0 and env.observation_space.high[174] == 1.0
    assert env.observation_space.low[175] == 0.0 and env.observation_space.high[175] == 1.0
    assert env.observation_space.low[176] == 0.0 and env.observation_space.high[176] == 1.0
    assert env.observation_space.low[177] == 0.0 and env.observation_space.high[177] == 6.0
    assert env.observation_space.low[178] == 0.0 and env.observation_space.high[178] == 1.0
    assert env.observation_space.low[179] == 0.0 and env.observation_space.high[179] == 1.0
    assert env.observation_space.low[180] == 0.0 and env.observation_space.high[180] == 1.0
    assert env.observation_space.low[181] == 0.0 and env.observation_space.high[181] == 6.0
    assert env.observation_space.low[182] == 0.0 and env.observation_space.high[182] == 1.0
    assert env.observation_space.low[183] == 0.0 and env.observation_space.high[183] == 1.0
    assert env.observation_space.low[184] == 0.0 and env.observation_space.high[184] == 1.0
    assert env.observation_space.low[185] == 0.0 and env.observation_space.high[185] == 6.0
    assert env.observation_space.low[186] == 0.0 and env.observation_space.high[186] == 1.0
    assert env.observation_space.low[187] == 0.0 and env.observation_space.high[187] == 1.0
    assert env.observation_space.low[188] == 0.0 and env.observation_space.high[188] == 1.0
    assert env.observation_space.low[189] == 0.0 and env.observation_space.high[189] == 6.0
    assert env.observation_space.low[190] == 0.0 and env.observation_space.high[190] == 1.0
    assert env.observation_space.low[191] == 0.0 and env.observation_space.high[191] == 1.0
    assert env.observation_space.low[192] == 0.0 and env.observation_space.high[192] == 1.0
    assert env.observation_space.low[193] == 0.0 and env.observation_space.high[193] == 6.0
    assert env.observation_space.low[194] == 0.0 and env.observation_space.high[194] == 7.5
    assert env.observation_space.low[195] == 0.0 and env.observation_space.high[195] == 1.0
    assert env.observation_space.low[196] == 0.0 and env.observation_space.high[196] == 1.0
    assert env.observation_space.low[197] == 0.0 and env.observation_space.high[197] == 1.0
